package timing.ukulele.auth.authorization.filter;

import com.nimbusds.jose.shaded.gson.internal.LinkedTreeMap;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.security.oauth2.jwt.JwtDecoder;
import org.springframework.util.StringUtils;
import org.springframework.web.filter.OncePerRequestFilter;
import timing.ukulele.auth.authorization.JwtAuthorizationToken;
import timing.ukulele.auth.security.model.CustomGrantedAuthority;

import java.io.IOException;
import java.time.Instant;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class JWTAuthorizationFilter extends OncePerRequestFilter {

    private final JwtDecoder jwtDecoder;

    public JWTAuthorizationFilter(JwtDecoder jwtDecoder) {
        this.jwtDecoder = jwtDecoder;
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        String token = request.getParameter("token");
        if (!StringUtils.hasText(token)) {
//          从请求头中获取认证信息
            String authHeader = request.getHeader("Authorization");
            if (authHeader == null || !authHeader.startsWith("Bearer ")) {
                filterChain.doFilter(request, response);
                return;
            }
            token = authHeader.substring(7);
        }
        if (!StringUtils.hasLength(token)) {
            filterChain.doFilter(request, response);
            return;
        }
        Jwt decode = jwtDecoder.decode(token);
        if (decode == null || decode.getExpiresAt() == null || decode.getExpiresAt().compareTo(Instant.now()) <= 0) {
            filterChain.doFilter(request, response);
            return;
        }
        // 将UserDetails存储到SecurityContextHolder中
        Set<CustomGrantedAuthority> authorityList = new HashSet<>();
        List<String> scopeList = decode.getClaimAsStringList("scope");
        for (String scope : scopeList) {
            CustomGrantedAuthority auth = new CustomGrantedAuthority(scope);
            authorityList.add(auth);
        }
        LinkedTreeMap<String, Object> userInfo = decode.getClaim("userInfo");
        User user = new User(decode.getClaimAsString("sub"), "", true, true, true, true, authorityList);
        JwtAuthorizationToken authenticationToken = new JwtAuthorizationToken(user, authorityList);
        SecurityContextHolder.getContext().setAuthentication(authenticationToken);
        filterChain.doFilter(request, response);
    }
}
