package timing.ukulele.auth.security.type.image;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.lang.Nullable;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.util.StringUtils;
import timing.ukulele.auth.security.LoginTypeEnum;

public class ImageCodeAuthenticationFilter extends AbstractAuthenticationProcessingFilter {
    private boolean postOnly = true;
    private final String usernameParameter = "username";
    private final String passwordParameter = "password";
    private final String captchaParameter = "captcha";
    private final String captchaIdParameter = "captchaId";

    public ImageCodeAuthenticationFilter() {
        super(new AntPathRequestMatcher(LoginTypeEnum.IMAGE_CODE.getPath(), "POST"));
    }

    public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response) throws AuthenticationException {
        if (this.postOnly && !request.getMethod().equals("POST")) {
            throw new AuthenticationServiceException("Authentication method not supported: " + request.getMethod());
        } else {
            String captcha = this.obtainCaptcha(request);
            captcha = captcha != null ? captcha : "";
            String captchaId = this.obtainCaptchaId(request);
            captchaId = captchaId != null ? captchaId : "";
            if (!(StringUtils.hasLength(captcha) && StringUtils.hasLength(captchaId))) {
                throw new AuthenticationServiceException("参数错误");
            } else {
                // TODO 校验图形验证码
            }
            String username = this.obtainUsername(request);
            username = username != null ? username.trim() : "";
            String password = this.obtainPassword(request);
            password = password != null ? password : "";
            if (!(StringUtils.hasLength(username) && StringUtils.hasLength(password))) {
                throw new AuthenticationServiceException("参数错误");
            }
            ImageCodeAuthenticationToken authRequest = new ImageCodeAuthenticationToken(username, password);
            this.setDetails(request, authRequest);
            return this.getAuthenticationManager().authenticate(authRequest);
        }
    }


    protected String obtainPassword(HttpServletRequest request) {
        return request.getParameter(this.passwordParameter);
    }

    protected String obtainUsername(HttpServletRequest request) {
        return request.getParameter(this.usernameParameter);
    }

    protected String obtainCaptcha(HttpServletRequest request) {
        return request.getParameter(this.captchaParameter);
    }

    protected String obtainCaptchaId(HttpServletRequest request) {
        return request.getParameter(this.captchaIdParameter);
    }

    protected void setDetails(HttpServletRequest request, ImageCodeAuthenticationToken authRequest) {
        authRequest.setDetails(this.authenticationDetailsSource.buildDetails(request));
    }

    public void setPostOnly(boolean postOnly) {
        this.postOnly = postOnly;
    }
}
