package timing.ukulele.auth.security.type.image;

import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.SecurityConfigurer;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.DefaultSecurityFilterChain;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.security.web.context.SecurityContextRepository;
import timing.ukulele.auth.security.handler.LoginFailureHandler;
import timing.ukulele.auth.security.handler.LoginSuccessHandler;
import timing.ukulele.auth.security.service.TimingUserDetailService;

public class ImageCodeAuthenticationSecurityConfig implements SecurityConfigurer<DefaultSecurityFilterChain, HttpSecurity> {

    private final LoginFailureHandler loginFailHandler;
    private final LoginSuccessHandler loginSuccessHandler;
    private final TimingUserDetailService userService;
    private final PasswordEncoder passwordEncoder;

    public ImageCodeAuthenticationSecurityConfig(LoginFailureHandler loginFailHandler,
                                                 LoginSuccessHandler loginSuccessHandler,
                                                 TimingUserDetailService userService,
                                                 PasswordEncoder passwordEncoder) {
        this.loginFailHandler = loginFailHandler;
        this.loginSuccessHandler = loginSuccessHandler;
        this.userService = userService;
        this.passwordEncoder = passwordEncoder;
    }


    @Override
    public void init(HttpSecurity http) {

    }

    @Override
    public void configure(HttpSecurity http) {
        ImageCodeAuthenticationFilter authenticationFilter = new ImageCodeAuthenticationFilter();
        AuthenticationManager authenticationManager = http.getSharedObject(AuthenticationManager.class);
        authenticationFilter.setAuthenticationManager(authenticationManager);
        authenticationFilter.setAuthenticationSuccessHandler(loginSuccessHandler);
        authenticationFilter.setAuthenticationFailureHandler(loginFailHandler);

        ImageCodeAuthenticationProvider authenticationProvider = new ImageCodeAuthenticationProvider(userService, passwordEncoder);

        http.authenticationProvider(authenticationProvider)
                .addFilterAt(authenticationFilter, UsernamePasswordAuthenticationFilter.class);
    }
}