package timing.ukulele.auth.security.type.wechat;

import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.SecurityConfigurer;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.web.DefaultSecurityFilterChain;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.security.web.context.SecurityContextRepository;
import timing.ukulele.auth.security.handler.LoginFailureHandler;
import timing.ukulele.auth.security.handler.LoginSuccessHandler;
import timing.ukulele.auth.security.service.TimingUserDetailService;

public class ThirdQrAuthenticationSecurityConfig implements SecurityConfigurer<DefaultSecurityFilterChain, HttpSecurity> {

    private final LoginFailureHandler loginFailHandler;
    private final LoginSuccessHandler loginSuccessHandler;
    private final TimingUserDetailService userService;

    public ThirdQrAuthenticationSecurityConfig(
            LoginFailureHandler loginFailHandler,
            LoginSuccessHandler loginSuccessHandler,
            TimingUserDetailService userService) {
        this.loginFailHandler = loginFailHandler;
        this.loginSuccessHandler = loginSuccessHandler;
        this.userService = userService;
    }


    @Override
    public void init(HttpSecurity http) {

    }

    @Override
    public void configure(HttpSecurity http) {
        ThirdQrAuthenticationFilter authenticationFilter = new ThirdQrAuthenticationFilter();
        AuthenticationManager authenticationManager = http.getSharedObject(AuthenticationManager.class);
        authenticationFilter.setAuthenticationManager(authenticationManager);
        authenticationFilter.setAuthenticationSuccessHandler(loginSuccessHandler);
        authenticationFilter.setAuthenticationFailureHandler(loginFailHandler);
        ThirdQrAuthenticationProvider authenticationProvider = new ThirdQrAuthenticationProvider();
        authenticationProvider.setUserService(userService);

        http.authenticationProvider(authenticationProvider)
                .addFilterAt(authenticationFilter, UsernamePasswordAuthenticationFilter.class);
    }
}